# -*- coding: utf-8 -*-

# 바람직한 스타일이 아닌 번역 표현을 찾아낸다. 특히 각종 번역체.

import string,re
from KPC.classes import Error, BaseCheck

R = re.compile

PLURAL_JOSA_RE = '(도|로|만|에|은|을|이|의)'
# 많이 등장하는 명사만 쓴다
PLURAL_NOUN_RE = '(값|개발자|것|기능|기술|그림|글자|답변|디렉터리|메시지|문서|문자|분|사람|사진|설정|언어|점|질문|파일|팀|패키지|폴더|프로그램|플러그인|항목|활동)'
PLURAL_EXCEPT = '(' + '핸들' + PLURAL_JOSA_RE + ')'

SYLLABLE_WITH_T_RIEUL = '[%s]' % ''.join([chr(c) for c in
                                          range(0xAC00 + 8, 0xD7A4, 28)])
SYLLABLE_WITH_T_SSANGSIOS = '[%s]' % ''.join([chr(c) for c in
                                              range(0xAC00 + 0x14, 0xD7A4, 28)])

data = [
    # 임의의 복수형 표현을 찾아내기는 매우 힘들다. 다음 휴리스틱으로만 검사.
    # (0) (들)이라고 쓰여진 경우
    # (1) 일단 알려진 명사에 대한 복수형태를 찾고
    # (2) 들+조사 형태로 끝나는 경우를 찾는다
    { 're': R('(' +
              '[\uac00-\ud7a3]+\(들\)' + '|' +
              PLURAL_NOUN_RE + '들' + '|' +
              '[\uac00-\ud7a3]+들' + PLURAL_JOSA_RE + '\s' + ')'),
      'error': '%s: 불필요한 복수형 표현',
      'except': R(PLURAL_EXCEPT),
      },

    # ...하기 위하여/위해 => ...하려면
    { 're': R('[\uac00-\ud7a3]+기 위(하여|해)'),
      'error': '%s: 어색한 표현. "-하려면"과 같이 쓰십시오.',
    },

    # ...와 함께
    { 're': R('\S+와 함께'),
      'error': '%s: 어색한 표현. 풀어서 써 보십시오.',
    },

    # ...에 의해
    { 're': R('\S+에 의해'),
      'error': '%s: 어색한 표현. 수동태 문장을 바꿔 보십시오.',
    },

    # 필요로 하다
    { 're': R('필요로 하\S+'),
      'error': '%s: 어색한 표현. "필요하다"와 같이 간결히 쓰십시오.',
    },

    # 필요가 있다
    { 're': R('필요가 있\S+'),
      'error': '%s: 어색한 표현. "...해야 합니다"와 같이 간결히 쓰십시오.',
    },

    # ...할 것입니다
    { 're': R('\S+' + SYLLABLE_WITH_T_RIEUL + ' 것입니다'),
      'error': '%s: 어색한 표현. "...합니다"와 같이 간결히 쓰십시오.',
      # 과거형인 "-했을 것입니다"는 예외
      'except': R('\S+' + SYLLABLE_WITH_T_SSANGSIOS + '을 것입니다'),
    },

    # ...하시기 바랍니다 것입니다
    { 're': R('\S+시기 바랍니다'),
      'error': '%s: 어색한 표현. "...하십시오"와 같이 간결히 쓰십시오.',
    },

    # "N개의 아무개"
    { 're': R('%\S*[du]개의 \S+'),
      'error': '%s: 어색한 표현. "몇개의 아무개" 대신 "아무개 몇개"와 같이 쓰십시오.',
    },

    # ...을 포함
    { 're': R('\S+[를을이가] 포함\S+'),
      'error': '%s: 어색한 표현. "..이 들어 있다"와 같이 바꿔 쓰십시오.',
    },
]

class BadStyleCheck(BaseCheck):
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        errors = []
        for entry in data:
            r = entry['re']
            errmsg = entry['error']
            mo = r.search(msgstr)
            if mo:
                if ('except' in entry) and entry['except'].match(mo.group(0)):
                    continue
                errors.append(Error(errmsg % mo.group(0)))
        return errors

name = 'language/badstyle'
description = '번역체 등 바람직하지 못한 스타일의 번역을 찾아냅니다'
checker = BadStyleCheck()



# -*- coding: utf-8 -*-
# Docbook XML markup check

from xml.parsers.expat import ParserCreate
from xml.parsers.expat import ExpatError
from KPC.classes import Error, BaseCheck

# 메시지에 대해 XML 파서를 돌려 well formed인지 검사하고, 태그가 알려진
# DocBook 태그인지 검사한다.

# DocBook XML의 태그는 수백 가지에 달하지만, 결벽증 마냥 모든 걸 나열할
# 필요는 없고 실제 발견될 때마다 추가하도록 한다.

known_db_tags = set(['acronym', 'address', 'anchor', 'application', 'citerefentry', 'citetitle', 'city', 'classname', 'command', 'computeroutput', 'country', 'email', 'emphasis', 'entry', 'envar', 'filename', 'firstterm', 'footnote', 'footnoteref', 'glossterm', 'guibutton', 'guiicon', 'guilabel', 'guimenu', 'guimenuitem', 'guisubmenu', 'indexterm', 'informalexample', 'informaltable', 'interface', 'interfacename', 'itemizedlist', 'keycap', 'keycombo', 'keysym', 'link', 'listitem', 'literal', 'manvolnum', 'menuchoice', 'option', 'orderedlist', 'para', 'phrase', 'postcode', 'primary', 'prompt', 'quote', 'refentrytitle', 'remark', 'replaceable', 'row', 'screen', 'screen', 'secondary', 'see', 'sgmltag', 'shortcut', 'state', 'street', 'subscript', 'superscript', 'systemitem', 'tbody', 'term', 'tertiary', 'tgroup', 'thead', 'title', 'trademark', 'ulink', 'uri', 'userinput', 'variablelist', 'varlistentry', 'varname', 'xref'])

tag_error_string = 'XML 태그의 짝이 맞지 않습니다'
notdb_error_string = '<%s>: 알려진 DocBook 태그가 아닙니다'

class NotDocBook(Exception):
    pass

def check_db_tags(name):
    if name == 'KPC_DummyTag':
        pass
    elif name[:12] == 'placeholder-':
        # old gnome-doc-utils magic
        pass
    elif name[:7] == '_:item-' or name[:7] == '_:link-':
        # newer gnome-doc-utils magic
        pass
    elif not name in known_db_tags:
        raise NotDocBook(name)

def start_element(name,attr):
    check_db_tags(name)

def end_element(name):
    check_db_tags(name)

class MarkupDocbookCheck(BaseCheck):
    def check(self, entry):
        if not entry.references or not '.xml:' in entry.references[0]:
            # not from an XML file
            return []
        msgid = entry.msgid
        msgstr = entry.msgstr
        if msgid == 'translator-credits': # gnome-doc-utils magic
            return []
        parser = ParserCreate()
        parser.StartElementHandler = start_element
        parser.EndElementHandler = end_element
        parser.UseForeignDTD(True)
        try:
            parser.Parse('<KPC_DummyTag>' + msgstr + '</KPC_DummyTag>')
        except ExpatError as e:
            return [Error(tag_error_string)]
        except NotDocBook as e:
            return [Error(notdb_error_string % e)]
        return []

name = 'convention/markup-docbook'
description = 'DocBook 마크업의 짝이 맞는지 검사합니다'
checker = MarkupDocbookCheck()



# -*- coding: utf-8 -*-

import re
from KPC.classes import Error, BaseCheck

data = [
    { 're': re.compile('.*[^\.]\.$'),
      'except': re.compile('.*(\)|etc|No|a\.m|p\.m)\.$'),
      'error':  Error('번역문이 원문과 같이 .으로 끝나야 합니다') },
    { 're': re.compile('.*:$'),
      'error':  Error('번역문이 원문과 같이 :으로 끝나야 합니다') },
    { 're': re.compile('.*[^\s]\.\.\.$'),
      'except': re.compile('.*(etc)\.\.\.$'),
      'error':  Error('번역문이 원문과 같이 ...으로 끝나야 합니다') },
    { 're': re.compile('.*…$'),
      'error':  Error('번역문이 원문과 같이 …으로 끝나야 합니다') },
    ]
    
class ConsistencyCheck(BaseCheck):
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        errors = []
        for d in data:
            re = d['re']
            error = d['error']
            if (re.match(msgid) and
                ('except' not in d or not d['except'].match(msgid)) and
                not re.match(msgstr)):
                errors.append(error)
        return errors

name = 'language/consistency'
description = '번역문이 원문과 비슷한 문장 부호로 끝나도록 합니다'
checker = ConsistencyCheck()



# -*- coding: utf-8 -*-

import string,re
from KPC.classes import Error, BaseCheck

# 의존명사를 위한 -할 형태의 동사 모음
verbs_re = '('+'|'.join([
    '가져올',
    '갈',
    '걸',
    '고칠'
    '나타날',
    '내릴',
    '놓일',
    '되돌릴',
    '만들',
    '바꿀',
    '보낼',
    '볼',
    '생길',
    '시킬',
    '쓸',
    '알',
    '얻어낼',
    '얻을',
    '없앨',
    '열',
    '올',
    '일',
    '읽어들일',
    '읽을',
    '있을',
    '잠글',
    '지울',
    '찾을',
    '할',
    ])+')'

variations_re = '(있다|없다|있습니다|없습니다|있는|없는|있게|없게)'

# 조사 모음
josa_re = '('+'|'.join([
    '가','이','이\(가\)','가\(이\)','\(이\)가','\(가\)이',
    '를','을','을\(를\)','를\(을\)','\(을\)를','\(를\)을',
    '는','은','은\(는\)','는\(은\)','\(은\)는','\(는\)은',
    '와','과','와\(과\)','과\(와\)',
    '로','으로','\(으\)로',
    '로서','으로서','\(으\)로서',
    '로써','으로써','\(으\)로써',
    '로부터','으로부터','\(으\)로부터',
    '라는','이라는','\(이\)라는',
    '의', '도', '에', '에서', '만', '부터',
    ])+')'

def test_noun_suffix(str):
    if (str[-1] != '슴'):
        return 1
    elif ((ord(str[-2]) - 0xac00) % 28 == 20):
        return 0
    elif (str[-2] == '없'):
        return 0
    else:
        return 1
    

misspell_data = [
    { 're':    re.compile('(않\s*(한|함|합니다|된|됨|됩니다))'),
      'error': '\"%s\": 짧은 부정문에서는 \'않\'이 아니라 \'안\'을 씁니다' },
    { 're':    re.compile('(읍니다)'),
      'error': '\"%s\": \'읍니다\'가 아니라 \'습니다\'입니다' },
    { 're':    re.compile('([^\s]+슴)([\s\.\,\?]|$)'),
      'func':  test_noun_suffix,
      'error': '\"%s\": \'슴\'이 아니라 \'음\'입니다' },
    { 're':    re.compile('('+verbs_re+'수(가|도|는)?\s'+')'),
      'error': '\"%s\": 의존 명사는 띄어 써야 합니다' },
    { 're':    re.compile('('+verbs_re+'\s*수(있|없)'+')'),
      'error': '\"%s\": 의존 명사는 띄어 써야 합니다' },
    { 're':    re.compile('('+verbs_re+'때(가|도|는)?\s'+')'),
      'error': '\"%s\": \'~할 때\'라고 띄어 써야 합니다' },
    { 're':    re.compile('([0-9A-Za-z-+\`\'\"()%_]+ '+josa_re+')\s'),
      'error': '\"%s\": 조사는 체언에 붙여 써야 합니다' },
]

class SpellCheck(BaseCheck):
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        errors = []
        for data in misspell_data:
            misspell_re = data['re']
            misspell_error = data['error']
            s = msgstr
            while 1:
                mo = misspell_re.search(s)
                if mo:
                    if ('func' in data and data['func'](mo.group(1))):
                        s = s[mo.end():]
                        continue
                    ret = 0
                    errors.append(Error(misspell_error % mo.group(1)))
                    s = s[mo.end():]
                else:
                    break;
        return errors

name = 'language/spell'
description = '흔히 하는 맞춤법 오류를 찾아냅니다'
checker = SpellCheck()



class Error:
    '''An individual error from a check against an entry. One check
    usually returns list of errors or empty list if there's no error.
    '''
    SUCCESS = 0
    DEPRECATED = 1
    WARNING = 2
    ERROR = 3

    def __init__(self, message):
        self.pos = 0
        self.level = Error.ERROR
        self.message = message

    def message(self):
        return message


class BaseCheck:
    '''Base class to check an entry from PO files. Subclasses should
    implement check() function. check() returns list of errors if any.
    '''
    def check(self, entry):
        return []


class CheckList(BaseCheck):
    '''Container class to hold other check classes. check() returns
    all errors from the contained sub-checks.
    '''
    def __init__(self, l=[]):
        self.checks = l

    def add(self, check):
        self.checks.append(check)

    def check(self, entry):
        ret = []
        for c in self.checks:
            ret += c.check(self, entry)
        return ret


class HeaderCheck(BaseCheck):
    '''Base class to check the header entries. Subclasses should
    implement check_header() function.
    '''
    def check(self, entry):
        # header messages only
        if entry.msgid != '':
            return []
        fields = {}
        # parse msgstr
        for line in entry.msgstr.split('\n'):
            try:
                k, v = line.split(': ', 1)
                fields[k] = v
            except ValueError:
                pass
        return self.check_header(entry, fields)



# -*- coding: utf-8 -*-

import re, string
from KPC.classes import Error, BaseCheck

re_accesskey_gnome = re.compile(r'^[^_]*_([0-9A-Za-z])[^_]*$')
re_accesskey_kde = re.compile(r'^[^&]*&([0-9A-Za-z])[^_]*$')

re_accesskey = re.compile(r'^[^_]*[_&]([0-9A-Za-z])[^_]*$')

# GNOME 예외적인 경우:
# - 소문자로 시작하는 경우
# - "abc_DEF" 등 중간에 밑줄 다음에 대문자가 나오는 경우
re_accesskey_gnome_unlikely = re.compile(r'([a-z].*|.*[a-zA-Z]_[A-Z].*)')

# KDE 예외적인 경우:
# - 소문자로 시작하는 경우
# - intltool에서 unescape되지 않은 XML character entity
# - URL처럼 보이는 경우
re_accesskey_kde_unlikely = re.compile(r'([a-z].*|.*(&quot;|&amp;|&apos;|&lt;|&gt;).*|.*[a-z]://\S+/\S+&[0-9a-zA-Z].*=)')

errstr_no_accesskey = '번역문에 접근키가 없거나 두 개 이상입니다'
errstr_wrong = '\'%s\' vs \'%s\': 원문과 번역문의 접근키가 다릅니다'

re_lowercase_accesskey = re.compile('(\([_&][a-z]\))')
errstr_lowercase = '\"%s\": 접근키가 소문자입니다'

class AccessKeyCheck(BaseCheck):
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        gnome_mo = re_accesskey_gnome.match(msgid)
        kde_mo = re_accesskey_kde.match(msgid)

        # doubts if the string really contains a access key
        if gnome_mo and re_accesskey_gnome_unlikely.match(msgid):
            gnome_mo = None
        if kde_mo and re_accesskey_kde_unlikely.match(msgid):
            kde_mo = None

        if gnome_mo or kde_mo:
            # too long string to be a label string
            if len(msgid) > 60:
                return []
            # GConf schema or GSettings schema file
            if entry.references and ('.schemas' in entry.references[0] or
                                     '.gschema' in entry.references[0]):
                return []

            if gnome_mo:
                letter = gnome_mo.group(1).upper()
            else:
                letter = kde_mo.group(1).upper()
            # check 1: if it's translated with the access key
            mo = re_accesskey.match(msgstr)
            if not mo:
                return [Error(errstr_no_accesskey)]

            # check 2: check if it's same
            if letter != mo.group(1).upper():
                return [Error(errstr_wrong % (letter, mo.group(1)))]

            # check 3: check if it's uppercase in "(_X)" form
            mo = re_lowercase_accesskey.search(msgstr)
            if mo:
                return [Error(errstr_lowercase % mo.group(1))]
        return []

name = 'convention/accesskey'
description = '접근키를 올바로 번역했는지 검사합니다'
checker = AccessKeyCheck()



# -*- coding: utf-8 -*-
# Mallard XML markup check

from xml.parsers.expat import ParserCreate
from xml.parsers.expat import ExpatError
from KPC.classes import Error, BaseCheck

# 메시지에 대해 XML 파서를 돌려 well formed인지 검사하고, 태그가 알려진
# Mallard 태그인지 검사한다.
# grep 'element name' mallard-1.0.rng | cut -f 2 -d'"' 
known_tags = set([
        'page',
        'section',
        'info',
        'credit',
        'link',
        'title',
        'license',
        'desc',
        'revision',
        'years',
        'email',
        'name',
        'links',
        'code',
        'example',
        'media',
        'p',
        'screen',
        'quote',
        'comment',
        'figure',
        'listing',
        'note',
        'synopsis',
        'list',
        'item',
        'steps',
        'item',
        'terms',
        'item',
        'tree',
        'item',
        'table',
        'col',
        'colgroup',
        'tr',
        'thead',
        'tbody',
        'tfoot',
        'td',
        'title',
        'subtitle',
        'desc',
        'cite',
        'app',
        'code',
        'cmd',
        'output',
        'em',
        'file',
        'gui',
        'guiseq',
        'link',
        'media',
        'keyseq',
        'key',
        'span',
        'sys',
        'input',
        'var',
])

tag_error_string = 'XML 태그의 짝이 맞지 않습니다'
unknown_tag_error_string = '<%s>: 알려진 Mallard 태그가 아닙니다'

class UnknownTag(Exception):
    pass

def check_tags(name):
    if name == 'KPC_DummyTag':
        pass
    elif name[:12] == 'placeholder-':
        # old gnome-doc-utils magic
        pass
    elif name[:7] == '_:item-' or name[:7] == '_:link-':
        # newer gnome-doc-utils magic
        pass
    elif not name in known_tags:
        raise UnknownTag(name)

def start_element(name,attr):
    check_tags(name)

def end_element(name):
    check_tags(name)

class MarkupMallardCheck(BaseCheck):
    def check(self, entry):
        if not entry.references or not '.page:' in entry.references[0]:
            # not from an XML file
            return []
        msgid = entry.msgid
        msgstr = entry.msgstr
        if msgid == 'translator-credits': # gnome-doc-utils magic
            return []
        parser = ParserCreate()
        parser.StartElementHandler = start_element
        parser.EndElementHandler = end_element
        parser.UseForeignDTD(True)
        try:
            parser.Parse('<KPC_DummyTag>' + msgstr + '</KPC_DummyTag>')
        except ExpatError as e:
            return [Error(tag_error_string)]
        except UnknownTag as e:
            return [Error(unknown_tag_error_string % e)]
        return []

name = 'convention/markup-mallard'
description = 'Mallard 마크업의 짝이 맞는지 검사합니다'
checker = MarkupMallardCheck()



# -*- coding: utf-8 -*-
# GNOME L10N administravia

import string
from KPC.classes import Error, HeaderCheck

class GnomeHeaderCheck(HeaderCheck):
    new_addr = '<gnome-kr@googlegroups.com>'
    old_addr = '<gnome-kr-hackers@lists.kldp.net>'
    error = Error('새 번역팀 주소를 사용하십시오: %s' % new_addr)

    def check_header(self, entry, fields):
        try:
            value = fields['Language-Team']
            if self.old_addr in value:
                return [self.error]
            if value.startswith('GNOME Korea') and not self.new_addr in value:
                return [self.error]
        except KeyError:
            pass
        return []

name = 'convention/gnome-team'
description = '그놈 한국어 번역팀 관련 사항을 검사합니다'
checker = GnomeHeaderCheck()



# -*- coding: utf-8 -*-

from KPC.classes import Error, BaseCheck

# 알파벳 순서대로 유지하세요
data = [
        ('about', '정보', '대하여'),
        ('application', '프로그램', '애플리케이션'),
        ('application', '프로그램', '어플리케이션'),
        ('background', '배경', '바탕'),
        ('button', '단추', '버튼'),
        ('change', '바꾸기', '변경'),
        ('clear', '지우기', '비우기'),
        ('clear', '지우기', '삭제'),
        ('create', '만들기', '생성'),
        ('data', '데이터', '데이타'),
        ('delete', '삭제', '지우기'),
        ('desktop', '데스크톱', '데스크탑'),
        ('desktop', '바탕 화면', '바탕화면'),
        ('directory', '디렉터리', '디렉토리'),
        ('download', '다운로드', '내려 받'),
        ('download', '다운로드', '내려받'),
        ('edit', '편집', '고치기'),
        ('exit', '나가기', '종료'),
        ('filename', '파일 이름', '파일명'),
        ('filename', '파일 이름', '파일이름'),
        ('filesystem', '파일 시스템', '파일시스템'),
        ('filter', '필터', '거르개'),
        ('filter', '필터', '거르게'),
        ('find', '찾기', '검색'),
        ('focus', '포커스', '초점'),
        ('font family', '글꼴 계열', '가족'),
        ('font family', '글꼴 계열', '패밀리'),
        ('font', '글꼴', '폰트'),
        ('game', '게임', '놀이'),
        ('graphics', '그래픽', '그래픽스'),
        ('graphics', '그래픽', '그림'),
        ('history', '기록', '히스토리'),
        ('hostname', '호스트 이름', '호스트명'),
        ('input method', '입력기', '입력 방법'),
        ('input method', '입력기', '입력 방식'),
        ('install','설치','깔기'),
        ('install','설치','인스톨'),
        ('item','항목','아이템'),
        ('key', '키', '글쇠'),
        ('keyboard', '키보드', '글쇠판'),
        ('launcher', '실행 아이콘', '실행기'),
        ('license', '라이선스', '라이센스'),
        ('link', '링크', '바로 가기'),
        ('link', '링크', '바로가기'),
        ('locale', '로캘', '로케일'),
        ('e-mail', '전자메일', '이메일'),
        ('email', '전자메일', '이메일'),
        ('message', '메시지', '메세지'),
        ('modify', '수정', '고치'),
        ('modify', '수정', '바꾸'),
        ('non-free', '독점', '비자유'),
        ('password', '암호', '비밀 번호'),
        ('password', '암호', '비밀번호'),
        ('password', '암호', '열쇠글'),
        ('paste', '붙여넣기', '붙여 넣기'),
        ('preferences', '기본 설정', '기본설정'),
        ('preferences', '기본 설정', '선택 사항'),
        ('preferences', '기본 설정', '설정 사항'),
        ('preferences', '기본 설정', '환경설정'),
        ('properties', '속성', '등록 정보'),
        ('remember password', '암호 저장', '암호 기억'),
        ('remove', '제거', '삭제'),
        ('remove', '제거', '지우기'),
        ('replace', '바꾸기', '고치'),
        ('replace', '바꾸기', '대체'),
        ('replace', '바꾸기', '치환'),
        ('restart', '다시 시작', '재시작'),
        ('screensaver', '화면 보호기', '화면보호기'),
        ('search', '검색', '찾기'),
        ('separator', '구분선', '구분자'),
        ('shortcut', '바로 가기', '단축키'),
        ('shortcut', '바로 가기', '바로가기'),
        ('software','소프트웨어','무른모'),
        ('template', '서식', '템플리트'),
        ('template', '서식', '템플릿'),
        ('template', '서식', '틀'),
        ('thread', '스레드', '쓰레드'),
        ('update', '업데이트', '갱신'),
        ('username', '사용자 이름', '사용자명'),
        ('version', '버전', '버젼'),
        ('version', '버전', '판번호'),
        ('windows', '윈도우', '윈도우즈'),
        ]

class WordGuideCheck(BaseCheck):
    errstr = '%s: \"%s\"은(는) \"%s\"(이)라고 번역하길 제안합니다'
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        errors = []
        msgid_l = msgid.replace('_','').replace('&','').lower()
        for (id, right, wrong) in data:
            if (wrong in msgstr) and (id in msgid_l) and ((right in wrong) or (not right in msgstr)):
                errors.append(Error(self.errstr % (wrong, id, right.strip())))
        return errors

name = 'terminology/word-guide'
description = '바람직하지 않은 번역 용어를 검사합니다'
checker = WordGuideCheck()



# -*- coding: utf-8 -*-

import string, re
from KPC.classes import Error, BaseCheck

typo = '('+'|'.join([
    '밍나합니다',
    '거싱',
    '살마',
    '[가-힣]빈다'
    ])+')'

typo_re = re.compile(typo)
typo_error = '\"%s\": 두벌식 오타로 보입니다'

class Typo2BulCheck(BaseCheck):
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        errors = []
        s = msgstr
        while 1:
            mo = typo_re.search(s)
            if mo:
                errors.append(Error(typo_error % mo.group(1)))
                s = s[mo.end():]
            else:
                break;
        return errors

name = 'language/typo2bul'
description = '두벌식 키보드를 사용할 때 발생할 수 있는 오타를 찾아냅니다'
checker = Typo2BulCheck()



# -*- coding: utf-8 -*-

# 그놈 프로그램에서 많이 쓰는 libegg에서 복사한 코드에 들어 있는 메시지
# 통일

from KPC.classes import Error, BaseCheck

data = [
    {'ref': 'eggdesktopfile.c',
     'msgid': 'File is not a valid .desktop file',
     'msgstr': '올바른 .desktop 파일이 아닙니다'},

    {'ref': 'eggdesktopfile.c',
     'msgid': 'Unrecognized desktop file Version \'%s\'',
     'msgstr': 'desktop 파일 버전을 (\'%s\') 인식할 수 없습니다'},

    {'ref': 'eggdesktopfile.c',
     'msgid': 'Starting %s',
     'msgstr': '%s 시작'},

    {'ref': 'eggdesktopfile.c',
     'msgid': 'Application does not accept documents on command line',
     'msgstr': '명령행에서 문서를 지정할 수 없는 프로그램입니다'},

    {'ref': 'eggdesktopfile.c',
     'msgid': 'Unrecognized launch option: %d',
     'msgstr': '알 수 없는 실행 옵션: %d'},

    {'ref': 'eggdesktopfile.c',
     'msgid': 'Can\'t pass document URIs to a \'Type=Link\' desktop entry',
     'msgstr': '문서 URI는 \'Type=Link\' desktop 항목에 넘길 수 없습니다'},

    {'ref': 'eggdesktopfile.c',
     'msgid': 'Not a launchable item',
     'msgstr': '실행할 수 있는 항목이 없습니다'},

    {'ref': 'eggsmclient.c',
     'msgid': 'Disable connection to session manager',
     'msgstr': '세션 관리자에 연결하지 않습니다'},

    {'ref': 'eggsmclient.c',
     'msgid': 'Specify file containing saved configuration',
     'msgstr': '설정을 저장할 파일을 지정합니다'},

    {'ref': 'eggsmclient.c',
     'msgid': 'FILE',
     'msgstr': '<파일>'},

    {'ref': 'eggsmclient.c',
     'msgid': 'Specify session management ID',
     'msgstr': '세션 관리 ID를 지정합니다'},

    {'ref': 'eggsmclient.c',
     'msgid': 'ID',
     'msgstr': '<ID>'},

    {'ref': 'eggsmclient.c',
     'msgid': 'Session management options:',
     'msgstr': '세션 관리 옵션:'},

    {'ref': 'eggsmclient.c',
     'msgid': 'Show session management options',
     'msgstr': '세션 관리 옵션을 표시합니다'},

    {'ref': 'egg-editable-toolbar.c',
     'msgid': 'Show “_%s”',
     'msgstr': '“_%s” 보이기'},

    {'ref': 'egg-editable-toolbar.c',
     'msgid': '_Move on Toolbar',
     'msgstr': '도구 모음에서 옮기기(_M)'},

    {'ref': 'egg-editable-toolbar.c',
     'msgid': 'Move the selected item on the toolbar',
     'msgstr': '도구 모음에서 선택한 항목을 옮깁니다'},

     {'ref': 'egg-editable-toolbar.c',
      'msgid': '_Remove from Toolbar',
      'msgstr': '도구 모음에서 제거(_R)'},

     {'ref': 'egg-editable-toolbar.c',
      'msgid': 'Remove the selected item from the toolbar',
      'msgstr': '도구 모음에서 선택한 항목을 제거합니다'},

     {'ref': 'egg-editable-toolbar.c',
      'msgid': '_Delete Toolbar',
      'msgstr': '도구 모음 삭제(_D)'},

     {'ref': 'egg-editable-toolbar.c',
      'msgid': 'Remove the selected toolbar',
      'msgstr': '선택한 도구 모음을 제거합니다'},

     {'ref': 'egg-editable-toolbar.c',
      'msgid': 'Separator',
      'msgstr': '구분선'},

    ]

class LibeggCheck(BaseCheck):
    errstr = '일관성을 위해 다음과 같이 번역합니다: %s'
    def check(self, entry):
        for d in data:
            ref = d['ref']
            if [x for x in entry.references if (d['ref']+':') in x]:
                if entry.msgid == d['msgid'] and entry.msgstr != d['msgstr']:
                    return [Error(self.errstr % d['msgstr'])]
        return []

name = 'terminology/libegg'
description = 'libegg에서 가져온 코드의 번역이 통일된 번역인지 검사합니다'
checker = LibeggCheck()



# -*- coding: utf-8 -*-
# terminology/common-labels

# 전형적인 번역문을 일관적으로 유지

import string, re
from KPC.classes import Error, BaseCheck

R = re.compile
def R(re_msgid, re_msgstr, label_msgid, label_msgstr):
    return (re.compile(re_msgid), re.compile(re_msgstr),
            label_msgid, label_msgstr)

data = [R('^View .*', '^.* 보기', 'View ...', '... 보기'),
        R('^Show .*', '^.* (보이기|표시)', 'Show ...', '... 보이기(표시)'),
        R('^Hide .*', '^.* 숨기기', 'Hide ...', '... 숨기기'),
       ]

def normalize_msg(msg):
    try:
        if msg[-3:] == '...':
            msg = msg[:-3]
        if msg[-4:-2] == '(_' and msg[-1:] == ')':
            msg = msg[:-4]
    except:
        pass
    msg = msg.replace('_', '')
    return msg

class CommonLabelsCheck(BaseCheck):
    errstr = '%s: \"%s\" 형태는 \"%s\" 형태와 같이 번역합니다'

    # 레이블이면 True
    def identify(self, entry):
        # 여러 줄 레이블 아님
        if '\n' in entry.msgid:
            return False
        # GConf schema or GSettings schema file
        if entry.references and ('.schemas' in entry.references[0] or
                                 '.gschema' in entry.references[0]):
            return False

        words = [x for x in entry.msgid.split(' ') if len(x)]
        # 모든 단어가 대문자로 시작하면 레이블
        if ''.join([x[0] for x in words]).isupper():
            return True
        # 단축키가 있으면 
        if '_' in entry.msgid:
            return True
        try:
            if entry.msgid[-3:] == '...':
                return True
        except:
            pass
        return False

    def check(self, entry):
        if not self.identify(entry):
            return []
        msgid = normalize_msg(entry.msgid)
        msgstr = normalize_msg(entry.msgstr)
        for (re_msgid, re_msgstr, label_msgid, label_msgstr) in data:
            if re_msgid.match(msgid):
                if not re_msgstr.match(msgstr):
                    return [Error(self.errstr %
                                  (entry.msgstr, label_msgid, label_msgstr))]
                else:
                    break
        return []

name = 'terminology/common-labels'
description = '전형적인 형태의 레이블을 일관적으로 번역했는지 검사합니다'
checker = CommonLabelsCheck()



# -*- coding: utf-8 -*-
import string

FUZZY,OBSOLETE,C_FORMAT,NO_C_FORMAT,PYTHON_FORMAT,NO_WRAP = 1,2,4,8,16,32

class entry:
    def __init__(self):
        self.msgid = ''
        self.msgid_plural = ''
        self.msgstr = ''
        self.msgctxt = ''
        self.translator_comment = ''
        self.automatic_comment = ''
        self.references = []
        self.flag = 0

    # attributes handling
    def set_flag(self,flag):
        self.flag = self.flag | flag
    def unset_flag(self,flag):
        self.flag = self.flag & ~flag
    def is_fuzzy(self):
        return (self.flag & FUZZY)
    def is_obsolete(self):
        return (self.flag & OBSOLETE)
    def is_untranslated(self):
        return (self.msgstr == '')
    def is_translated(self):
        return (not self.is_fuzzy() and
                not self.is_obsolete() and
                not self.is_untranslated())
    def is_c_format(self):
        return (self.flag & C_FORMAT)
    def is_no_c_format(self):
        return (self.flag & NO_C_FORMAT)
    def is_python_format(self):
        return (self.flag & PYTHON_FORMAT)
    def is_no_wrap(self):
        return (self.flag & NO_WRAP)
    def __repr__(self):
        return repr(self.msgid) + ':::' + repr(self.msgstr)

class catalog:
    def __init__(self):
        self.entries = []
        self.metadata = {}
        self.textdomain = ''
        self.language = 'ko'
    def add_entry(self,entry):
        if (entry.msgid == ''):         # header entry
            a = entry.msgstr.split('\n')
            for l in a:
                if len(l) == 0:
                    continue
                k, v = l.split(': ', 1)
                self.metadata[k] = v
        self.entries.append(entry)
    def settextdomain(self, d):
        self.textdomain = d



# -*- coding: utf-8 -*-

import string, re
from KPC.classes import Error, BaseCheck

class CopyrightCheck(BaseCheck):
    copyright_re = re.compile('^([Cc]opyright )?\([Cc]\) ')
    error = Error('copyright notice는 번역하면 안 됩니다')
    def check(self, entry):
        msgid_lines = entry.msgid.split('\n')
        msgstr_lines = entry.msgstr.split('\n')
        for (msgid, msgstr) in zip(msgid_lines, msgstr_lines):
            if self.copyright_re.match(msgid) and msgid != msgstr:
                return [self.error]
        return []

name = 'convention/copyright'
description = '저작권 표시에 대한 번역문을 검사합니다'
checker = CopyrightCheck()



# -*- coding: utf-8 -*-
'''Parse GNU gettext compliant PO (Portable Object) file.

'''

__all__ = ['parse_file', 'parse_entry', 'ParseError',
           'FUZZY', 'OBSOLETE', 'C_FORMAT', 'NO_C_FORMAT', 'PYTHON_FORMAT',
           'NO_WRAP']

import KPC.po as po
import re
import string

class ParseError(Exception):
    def __init__(self, lineno=-1):
        self.lineno = lineno
    def __str__(self):
        return 'lineno: %d' % self.lineno

def c2rawstring(str):
    return eval('"'+str+'"')

FUZZY,OBSOLETE,C_FORMAT,NO_C_FORMAT,PYTHON_FORMAT,NO_WRAP = 1,2,4,8,16,32

def parse_file(file):
    reader = file
    lineno = 0
    catalog = po.catalog()
    try:
        (entry,lineno) = parse_entry(reader,lineno)
    except ParseError:
        raise ParseError(lineno)
    catalog.add_entry(entry)
    content_type = catalog.metadata['Content-Type']
    try:
        charset = re.compile('charset=(.+)$').search(content_type).group(1)
        if charset == 'CHARSET':
            charset = 'UTF-8'
    except:
        charset = 'UTF-8'
    while 1:
        try:
            (entry,lineno) = parse_entry(reader,lineno)
        except ParseError as err:
            raise err
        if not entry:
            return catalog
        #try:
        #    entry.translator_comment = entry.translator_comment.decode(charset)
        #    entry.msgstr = entry.msgstr.decode(charset)
        #    entry.msgid = entry.msgid.decode(charset)
        #except:
        #    raise ParseError(lineno)
        catalog.add_entry(entry)

STATE_FIRST,STATE_COMMENT,STATE_ECOMMENT,STATE_MSGCTXT,STATE_MSGID,STATE_MSGSTR = 1,2,3,4,5,6
emptyline_re = re.compile(r'^\s*$')
translator_comment_re = re.compile(r'^\#( (.*))?$')
automatic_comment_re = re.compile(r'^\#. (.*)$')
reference_re = re.compile(r'^\#: (.*)$')
flag_re = re.compile(r'^\#, (.*)$')
string_re = re.compile(r'^\"(.*)\"\w*')

def read_string(fmt):
    try:
        str = string_re.match(fmt).group(1)
    except:
        raise ParseError
    return c2rawstring(str)

import codecs

def parse_entry(file,lineno):
    state = STATE_FIRST
    new_entry = po.entry()
    while 1:
        lineno += 1
        line = file.readline()
        if not line:                    # EOF
            if state == STATE_FIRST or state == STATE_COMMENT:
                return (None,lineno)    # no more messages -- return nothing
            elif state != STATE_MSGSTR:
                print('uhh: %s\n' % line)
                raise ParseError(lineno)        # unexpected EOF
            else:
                return (new_entry,lineno)
        if emptyline_re.match(line):
            if state == STATE_FIRST or state == STATE_COMMENT:
                continue
            elif state != STATE_MSGSTR:
                print('blah: %s\n' % line)
                raise ParseError(lineno)
            else:
                return (new_entry,lineno)
        if line[-1] == '\n':            # remove the trailing newline
            line = line[:-1]
        if line[:3] == '#~ ':
            new_entry.set_flag(OBSOLETE)
            line = line[3:]
        # comments
        if line[0] == '#':
            if state == STATE_FIRST:
                state = STATE_COMMENT
            if len(line) == 1:
                new_entry.translator_comment += '\n'
            elif line[1] == ' ':          # automatic comment
                new_entry.translator_comment += line[2:] + '\n'
            elif line[1] == ':':
                state = STATE_ECOMMENT
                new_entry.references += line[3:].split(' ')
            elif line[1] == ',':
                state = STATE_ECOMMENT
                for flag in line[3:].split(', '):
                    if flag == 'c-format':
                        new_entry.set_flag(C_FORMAT)
                    elif flag == 'no-c-format':
                        new_entry.set_flag(NO_C_FORMAT)
                    elif flag == 'python-format':
                        new_entry.set_flag(PYTHON_FORMAT)
                    elif flag == 'fuzzy':
                        new_entry.set_flag(FUZZY)
                    elif flag == 'no-wrap':
                        new_entry.set_flag(NO_WRAP)
                pass
            elif line[1] == '~':
                state = STATE_ECOMMENT
                pass
        else:
            if line[:9] == 'msgctxt "':
                state = STATE_MSGCTXT
                try:
                    new_entry.msgctxt += read_string(line[8:])
                except ParseError:
                    raise ParseError(lineno)
            elif line[:7] == 'msgid "':
                state = STATE_MSGID
                try:
                    new_entry.msgid += read_string(line[6:])
                except ParseError:
                    raise ParseError(lineno)
                new_entry.msgid_lineno = lineno
            elif line[:14] == 'msgid_plural "':
                state = STATE_MSGID
                new_entry.msgid_plural += read_string(line[13:])
            elif line[:8] == 'msgstr "':
                state = STATE_MSGSTR
                new_entry.msgstr += read_string(line[7:])
                new_entry.msgstr_lineno = lineno
            elif line[:7] == 'msgstr[':
                state = STATE_MSGSTR
                new_entry.msgstr += read_string(line[10:])
                new_entry.msgstr_lineno = lineno
            elif line[0] == '"':
                if state == STATE_MSGID:
                    new_entry.msgid += read_string(line)
                elif state == STATE_MSGSTR:
                    new_entry.msgstr += read_string(line)
                elif state == STATE_MSGCTXT:
                    new_entry.msgctxt += read_string(line)
                else:
                    raise ParseError(lineno)
            else:
                raise ParseError(lineno)
            
    #new_entry.msgid += line
    return (new_entry,lineno)
        
        
def test():
    import sys
    if sys.argv[1:]:
        fn = sys.argv[1]
        if fn == '-':
            fp = sys.stdin
        else:
            fp = open(fn, 'rb')
    else:
        import io
        fp = io.StringIO(test_input)
    catalog = parse_file(fp)
    print(str(catalog))

if __name__ == '__main__':
    test()



# -*- coding: utf-8 -*-
# 헤더 코멘트 검사

import string
from KPC.classes import Error, HeaderCheck

class HeaderCommentCheck(HeaderCheck):
    def check_header(self, entry, fields):
        errors = []
        comment = entry.translator_comment
        # PO-Revision-Date의 연도에 맞게 코멘트의 연도가 업데이트되었는지 확인
        try:
            name = fields['Last-Translator'].split(' <')[0].strip()
            year = fields['PO-Revision-Date'][:4]
        except IndexError:
            # FIXME: PO-Revision-Date 또는 Last-Translator가 포맷에 맞지 않는 경우, 또 다른 에러 출력?
            pass
        for line in comment.split('\n'):
            if name in line and year in line:
                break
        else:
            errors.append(Error('헤더 코멘트에 %s 번역자의 %s년도 기록이 없습니다' % (name, year)))
        return errors
        

name = 'convention/header-comment'
description = '헤더 코멘트가 올바른지 검사합니다'
checker = HeaderCommentCheck()



# -*- coding: utf-8 -*-
# markup translation
# Ref: http://developer.gnome.org/doc/API/2.0/pango/PangoMarkupFormat.html

import re, string
from KPC.classes import Error, BaseCheck

tags = [ 'span', 'b', 'big', 'i', 's', 'sub', 'sup', 'small', 'tt', 'u' ]
tags_res = '(' + '|'.join(tags) + ')'
re_markup = re.compile('<(' + tags_res + '( [^>]*)?)>')
group_opentag = 1
group_opentagname = 2

error_string = '<%s>: 번역할 때 마크업을 똑같이 써야 합니다'

class MarkupPangoCheck(BaseCheck):
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        mo = re_markup.search(msgid)
        while mo:
            tagname = mo.group(group_opentagname)
            opentag = mo.group(group_opentag)
            mo = re.search('<' + opentag + '>.+</' + tagname + '>', msgstr, re.M | re.S)
            if not mo:
                return [Error(error_string % opentag)]
            msgid = msgid[mo.start(0) + 1:]
            mo = re_markup.search(msgid)
        return []

name = 'convention/markup-pango'
description = '판고 마크업의 짝이 맞는지 검사합니다'
checker = MarkupPangoCheck()



# -*- coding: utf-8 -*-
# KDE specific rule

from KPC.classes import Error, BaseCheck

class KDECheck(BaseCheck):
    name_error = Error('KDE 번역 규칙에 따라 번역자 이름을 써야 합니다')
    email_error = Error('KDE 번역 규칙에 따라 번역자 메일 주소를 써야 합니다')
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        msgctxt = entry.msgctxt
        if msgctxt == 'NAME OF TRANSLATORS':
            # msgid "Your names"
            if '이름' in msgstr or '성함' in msgstr:
                return [self.name_error]
        elif msgctxt == 'EMAIL OF TRANSLATORS':
            # msgid "Your emails"
            if '메일' in msgstr or '편지' in msgstr:
                return [self.email_error]
        return []


name = 'convention/kde'
description = 'KDE 프로젝트에서 사용하는 관행에 맞는지 검사합니다'
checker = KDECheck()



# -*- coding: utf-8 -*-
# GNOME specific rule

from KPC.classes import Error, BaseCheck

class GnomeCheck(BaseCheck):
    error = Error('그놈 번역 규칙에 따라 번역자의 이름을 써야 합니다')
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        if ((msgid[:18] == 'translator_credits' or
             msgid[:18] == 'translator-credits') and
            (msgstr[:18] == 'translator_credits' or msgstr[:2] == '번역')):
            return [self.error]
        return []

name = 'convention/gnome'
description = '그놈 프로젝트에서 사용하는 관행에 맞는지 검사합니다'
checker = GnomeCheck()



# -*- coding: utf-8 -*-

import re,string
from KPC.classes import Error, BaseCheck

josa_data = [('가','이','이(가)'),
              ('를','을','을(를)'),
              ('는','은','은(는)'),
              ('와','과','과(와)'),
              ('로','으로','(으)로'),
              ('로서','으로서','(으)로서'),
              ('로써','으로써','(으)로써'),
              ('로부터','으로부터','(으)로부터'),
              ('라는','이라는','(이)라는')]
c_format = '%(?:[1-9][0-9]*\$)?[-+ #\'0]*(?:[1-9][0-9]*)?(?:\.[0-9]+)?(?:(?:hh|h|j|l|L|ll|q|t|z|Z)?[dioufeEgGaAcCspnmhjlLqtxXzZ1-9]|hh|ll)'
py_format = '%(?:\([A-Za-z]\w+\))?[#-0\ +]*(?:[0-9]+|\*)?(?:\.[0-9]+)?[hlL]?[diouxXeEfFgGcrs]'

josa = '(?:' + '|'.join([p[0]+'|'+p[1] for p in josa_data]) + ')'
josa_c_re = re.compile('(?P<case>(?P<fmt>'+c_format+'[\'\"]?) ?(?P<josa>'+josa+'))(?:\s|$)')
josa_py_re = re.compile('(?P<case>(?P<fmt>'+py_format+'[\'\"]?) ?(?P<josa>'+josa+'))(?:\s|$)')

def josa_suggest(cho):
    for (a,b,s) in josa_data:
        if cho == a or cho == b:
            return s
    return '????'

class JosaAlternativeCheck(BaseCheck):
    errstr = '\"%s\": 받침에 따른 조사 구별이 없습니다. \'%s\''
    def check(self, entry):
        if entry.is_c_format():
            josa_re = josa_c_re
        elif entry.is_python_format():
            josa_re = josa_py_re
        else:
            return []
        msgid = entry.msgid
        msgstr = entry.msgstr
        errors = []
        while True:
            mo = josa_re.search(msgstr)
            if mo:
                sug = mo.group('fmt') + josa_suggest(mo.group('josa'))
                errors.append(Error(self.errstr % (mo.group('case'), sug)))
                msgstr = msgstr[mo.end():]
            else:
                break;
        return errors

name = 'language/josa-alternative'
description = '조사 구분을 했는지 검사합니다'
checker = JosaAlternativeCheck()



#!/usr/bin/env python

import sys
if sys.version_info.major < 3:
    print('This program is only for Python 3.x. Run with python3.')
    sys.exit(1)

from distutils.core import setup
from KPC.config import VERSION

setup(name='ko-po-check',
      version=VERSION,
      description='Korean PO checker',
      author='Changwoo Ryu',
      author_email='cwryu@debian.org',
      url='https://github.com/changwoo/ko-po-check',

      packages = ['KPC', 'KPC/checks',
                  'KPC/checks/language',
                  'KPC/checks/convention',
                  'KPC/checks/terminology' ],
      scripts=['scripts/ko-po-check'],
      classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ' +
            'GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: Korean',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Localization',
        ],
      )
 



# -*- coding: utf-8 -*-

import string, re
from KPC.classes import Error, BaseCheck

# 괄호 띄어쓰기
check_list = [
    [ re.compile('[\uac00-\ud7a3] \(\S', re.UNICODE), '맞춤법 규정에 따라 괄호 앞에 띄어 쓰지 않습니다' ]
]

class PunctuationCheck(BaseCheck):
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        errors = []
        for (r, emsg) in check_list:
            mo = r.search(msgstr)
            if mo:
                errors.append(Error(emsg))
        return errors

name = 'language/punctuation'
description = '잘못된 한국어 구문 규칙을 찾아냅니다'
checker = PunctuationCheck()



# -*- coding: utf-8 -*-
# header checks

import string
from KPC.classes import Error, HeaderCheck

class CommonHeaderCheck(HeaderCheck):
    def check_header(self, entry, fields):
        result = []
        # Report-Msgid-Bugs-To: 내용이 없는 경우
        try:
            if fields['Report-Msgid-Bugs-To'] == '':
                errmsg = 'Report-Msgid-Bugs-To: 비어 있습니다. 버그 보고하는 메일 혹은 웹 주소를 쓰십시오'
                result.append(Error(errmsg))
        except KeyError:
            pass
        # Language-Team: 쓰지 않았거나 gettext 기본값을 바꾸지 않은 경우
        if 'Language-Team' not in fields:
            errmsg = 'Language-Team: 필드가 없습니다. 번역 팀이 따로 없으면 본인 주소를 쓰십시오'
            result.append(Error(errmsg))
        elif fields['Language-Team'] == '':
            errmsg = 'Language-Team: 비어 있습니다. 번역 팀이 따로 없으면 본인 주소를 쓰십시오'
            result.append(Error(errmsg))
        elif 'ko@li.org' in fields['Language-Team']:
            errmsg = 'Language-Team: 기본값을 바꾸지 않았습니다. 해당 번역팀 메일 주소를 쓰십시오'
            result.append(Error(errmsg))
        # Content-Type: UTF-8 추천
        if (not 'charset=UTF-8' in fields['Content-Type'] and
            not 'charset=utf-8' in fields['Content-Type']):
            errmsg = 'Content-Type: UTF-8 사용을 추천합니다 (charset=UTF-8)'
            result.append(Error(errmsg))
        # Plural-Forms: 한국어에 대한 복수형이 아닌 경우
        correct_value = 'nplurals=1; plural=0;'
        try:
            if fields['Plural-Forms'] != correct_value:
                errmsg = 'Plural-Forms: 한국어는 다음과 같이 씁니다: "%s"'
                result.append(Error(errmsg % correct_value))
        except KeyError:
            pass
        return result

name = 'convention/common-header'
description = '헤더 메시지가 올바른지 검사합니다'
checker = CommonHeaderCheck()



# -*- coding: utf-8 -*-
# terminology/desktop-commons

# UI 나타나는 전형적인 단어들의 전형적인 번역 확인.  지정한 번역이
# 아니면 애러를 리턴한다.  번역 일관성 유지를 위한 검사.

import string
from KPC.classes import Error, BaseCheck

data = [('about', '정보'),
        ('add', '추가'),
        ('always on top', '항상 위'),
        ('browse', '찾아보기'),
        ('change', '바꾸기'),
        ('clear', '지우기'),
        ('copy', '복사'),
        ('create', '만들기'),
        ('cut', '잘라내기'),
        ('delete', '삭제'),
        ('display', '표시'),
        ('edit', '편집'),
        ('exit', '나가기'),
        ('find', '찾기'),
        ('go', '이동'),
        ('hide', '숨기기'),
        ('location', '위치'),
        ('log in', '로그인'),
        ('log out', '로그아웃'),
        ('modify', '수정'),
        ('name', '이름'),
        ('next', '다음'),
        ('no', '아니요'),
        ('open', '열기'),
        ('paste', '붙여넣기'),
        ('preferences', '기본 설정'),
        ('previous', '이전'),
        ('properties', '속성'),
        ('quit', '끝내기'),
        ('refresh', '새로 고침'),
        ('reload', '다시 읽기'),
        ('remove', '제거'),
        ('replace', '바꾸기'),
        ('run', '실행'),
        ('save as', '다른 이름으로 저장'),
        ('save', '저장'),
        ('search', '검색'),
        ('show', '보이기'),
        ('statusbar', '상태 표시줄'),
        ('template', '서식'),
        ('view', '보기'),
        ('zoom in', '확대'),
        ('zoom out', '축소'),
        ]

def normalize_msgid(msgid):
    msgid = msgid.replace('_', '')
    if msgid[-3:] == '...':
        msgid = msgid[:-3]
    return msgid

def make_msgstr(msgstr,mnemonic,dots):
    if mnemonic:
        msgstr = msgstr + '(_' + mnemonic + ')'
    if dots:
        msgstr = msgstr + '...'
    return msgstr

def find_mnemonic(msgid):
    try:
        i = msgid.index('_')
        return msgid[i+1].upper()
    except ValueError:
        return None
    
class DesktopCommonsCheck(BaseCheck):
    errstr = '%s: 다음과 같이 번역해야 합니다: \"%s\"'

    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        errors = []
        msgid_n = normalize_msgid(msgid)
        msgid_m = find_mnemonic(msgid)
        msgid_d = (msgid[-3:] == '...')
        for (orig, trans) in data:
            # 모두 대문자로 된 단어는 option argument 따위로 쓰이므로 넘어간다
            if msgid_n.lower() == orig and not msgid_n.isupper():
                good_msgstr = make_msgstr(trans, msgid_m, msgid_d)
                if good_msgstr != msgstr:
                    errors.append(Error(self.errstr % (msgstr, good_msgstr)))
        return errors

name = 'terminology/desktop-commons'
description = '많이 사용되는 데스크톱 용어를 일관적으로 번역했는지 검사합니다'
checker = DesktopCommonsCheck()



# -*- coding: utf-8 -*-

import sys
import string,re
from KPC.classes import Error, BaseCheck

typo_re = re.compile('([\u3131-\u318E]+)')
typo_exception_re = re.compile('^[\u3131-\u318E]-[\u3131-\u318E]') # gdm
typo_error = '\"%s\": 음절이 불완전합니다.  오타로 보입니다'

class TypoIncompleteCheck(BaseCheck):
    def check(self, entry):
        msgid = entry.msgid
        msgstr = entry.msgstr
        errors = []
        s = msgstr
        while 1:
            mo = typo_re.search(s)
            if mo and not typo_exception_re.match(s):
                errors.append(Error(typo_error % mo.group(1)))
                s = s[mo.end():]
            else:
                break;
        return errors

name = 'language/typoincomplete'
description = '오타로 보이는 불완전한 음절을 찾아냅니다'
checker = TypoIncompleteCheck()
